#!/usr/bin/perl
# j5_xml_rpc_client_design_downstream_automation.pl
# Nathan Hillson 01-SEP-15

# *** Copyright Notice ***
#
# Copyright (c) 2011-2015, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory (subject to receipt of
# any required approvals from the U.S. Dept. of Energy). All rights
# reserved.
#
# NOTICE. This software was developed under funding from the U.S. Department of Energy.
# As such, the U.S. Government has been granted for itself and others acting on its
# behalf a paid-up, nonexclusive, irrevocable, worldwide license in the Software to
# reproduce, prepare derivative works, and perform publicly and display publicly.
# Beginning five (5) years after the date permission to assert copyright is obtained
# from the U.S. Department of Energy, and subject to any subsequent five (5) year
# renewals, the U.S. Government is granted for itself and others acting on its behalf a
# paid-up, nonexclusive, irrevocable, worldwide license in the Software to reproduce,
# prepare derivative works, distribute copies to the public, perform publicly and
# display publicly, and to permit others to do so. 
#
# This program is free software; you can redistribute it and/or modify
# it under the same terms as the Perl "Artistic License"
# (http://dev.perl.org/licenses/artistic.html).
#
# *** End of Copyright Notice ***

use XML::RPC;
use MIME::Base64;
require LWP::UserAgent;

my $assembly_to_automate_filename;
my $automation_task;
my $downstream_automation_parameters_filename;
my $encoded_downstream_automation_parameters_file;
my $encoded_plate_list_file;
my $encoded_zipped_plate_files_file;
my $encoded_assembly_to_automate_file;
my $input_file_handle;
my $j5_session_id;
my $output_file_handle;
my $output_filename;
my $reuse_downstream_automation_parameters_file;
my $plate_list_filename;
my $result;
my $ua;
my $xmlrpc;
my $zipped_plate_files_filename;

if (@ARGV != 6) {
    die "Usage: $0 j5_session_id TRUE_or_DownstreamAutomationParametersFileName PlateListFileName ZippedPlateFilesFileName AssemblyToAutomateFileName DistributePcrReactions\n";
}

$j5_session_id = $ARGV[0];

# setting the user agent timeout to a longer period of time is required to make sure that j5 has sufficient time to complete an assembly
$ua = LWP::UserAgent->new();
$ua->timeout( 86400 ); # wait up until 24 hours for j5 to complete assembly, then bail
$xmlrpc = XML::RPC->new('https://j5.jbei.org/bin/j5_xml_rpc.pl',lwp_useragent => $ua);

# call the j5 xml-rpc service to see if last updated files are available
$result = $xmlrpc->call( 'GetLastUpdatedUserFiles', { j5_session_id => $j5_session_id } );

if (defined $result->{"faultString"}) {
  die "Error (GetLastUpdatedUserFiles): faultCode: ".$result->{"faultCode"}." faultString: ".$result->{"faultString"}."\n";
}

$username = $result->{"username"};

# initialize the parameters for calling the DownstreamAutomation method
# downstream automation parameters file
if ($ARGV[1]=~/^TRUE$/i) {
  if ($result->{"downstream_automation_parameters_file_exists"}=~/^TRUE$/i) {
    $reuse_downstream_automation_parameters_file="TRUE";
  }
  else {
    die "Error (DesignDownstreamAutomation): downstream automation parameters file not found on server for user $username";
  }
}
else {
  $reuse_downstream_automation_parameters_file="FALSE";

  $downstream_automation_parameters_filename = $ARGV[1];
  # encode the downstream_automation_parameters_file
  open($input_file_handle, "$downstream_automation_parameters_filename") or die "$!";
  while (read($input_file_handle, $input_buffer, 60*57)) {
    $encoded_downstream_automation_parameters_file.=encode_base64($input_buffer);
  }
  chomp $encoded_downstream_automation_parameters_file;
  close($input_file_handle);
}

$plate_list_filename = $ARGV[2];
$zipped_plate_files_filename = $ARGV[3];
$assembly_to_automate_filename = $ARGV[4];
$automation_task = $ARGV[5];

# encode the plate_list_file
open($input_file_handle, "$plate_list_filename") or die "$!";
while (read($input_file_handle, $input_buffer, 60*57)) {
  $encoded_plate_list_file.=encode_base64($input_buffer);
}
chomp $encoded_plate_list_file;
close($input_file_handle);

# encode the zipped_plate_files_file
open($input_file_handle, "$zipped_plate_files_filename") or die "$!";
while (read($input_file_handle, $input_buffer, 60*57)) {
  $encoded_zipped_plate_files_file.=encode_base64($input_buffer);
}
chomp $encoded_zipped_plate_files_file;
close($input_file_handle);

# encode the assembly_to_automate_file
open($input_file_handle, "$assembly_to_automate_filename") or die "$!";
while (read($input_file_handle, $input_buffer, 60*57)) {
  $encoded_assembly_to_automate_file.=encode_base64($input_buffer);
}
chomp $assembly_to_automate;
close($input_file_handle);

# call the j5 xml-rpc service to condense the assembly files
$result = $xmlrpc->call( 'DesignDownstreamAutomation', { 
							j5_session_id => $j5_session_id,
							reuse_downstream_automation_parameters_file => $reuse_downstream_automation_parameters_file,
							encoded_downstream_automation_parameters_file => $encoded_downstream_automation_parameters_file,
							encoded_plate_list_file => $encoded_plate_list_file,
							encoded_zipped_plate_files_file => $encoded_zipped_plate_files_file,
							encoded_assembly_to_automate_file => $encoded_assembly_to_automate_file,
							automation_task => $automation_task,
						       }
		       );

if (defined $result->{"faultString"}) {
  print "Error: faultCode: ",$result->{"faultCode"}," faultString: ",$result->{"faultString"},"\n";
}

else {
  if (defined $result->{"error_message"}) {
    print "Error: ",$result->{"error_message"},"\n";
  }

  $output_filename = $result->{"output_filename"};
  open($output_file_handle, ">$output_filename") or die "$!";
  print $output_file_handle decode_base64($result->{"encoded_output_file"});
  close($output_file_handle);
}
